<?php

namespace Drupal\gearbox\Extension;

use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ThemeHandler;
use Drupal\Core\Serialization\Yaml;

class HandlerFactory {

  /**
   * @param ThemeHandler $handler
   *
   * @return ThemeHandler
   */
  static public function themeAlter(ThemeHandler $handler): ThemeHandler {
    foreach ($handler->listInfo() as $theme_original) {
      if ($theme_original->module_dependencies['gearbox'] ?? NULL) {
        $theme = new Extension(DRUPAL_ROOT, $theme_original->getType(), $theme_original->getPathname(), $theme_original->getExtensionFilename());
        foreach (get_object_vars($theme_original) as $k => $v) {
          $theme->$k = $v;
        }
        $handler->addTheme($theme);
      }
    }
    return $handler;
  }

  /**
   * @param ModuleHandler $handler
   *
   * @return ModuleHandler
   */
  static public function moduleAlter(ModuleHandler $handler): ModuleHandler {
    $module_list = $handler->getModuleList();
    foreach ($module_list as $name => $module_original) {
      $info = Yaml::decode(file_get_contents($module_original->getPathname()));
      if($module_original->getName() === 'gearbox' || in_array('gearbox', $info["dependencies"] ?? [])) {
          $module_list[$name] = new Extension(DRUPAL_ROOT, $module_original->getType(), $module_original->getPathname(), $module_original->getExtensionFilename());
      }
    }
    $handler->setModuleList($module_list);
    return $handler;
  }
}
